@@ -1,12 +1,11 @@
 //
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 // SDL Video Code
@@ -25,19 +24,16 @@
 #include "video.h"
 #include "sw.h"
 #include "swinit.h"
+#include "swmain.h"
 
-// lcd mode to emulate my old laptop i used to play sopwith on :)
+#define INPUT_BUFFER_LEN 32
 
-//#define LCD
-
-static SDL_Color cga_pal[] = {{}, {}, {}, {}};
-
-typedef struct {
+struct palette {
 	char name[13]; // Up to 12 characters will display correctly on the menu
 	SDL_Color color[4];
-} VideoPalette;
+};
 
-VideoPalette VideoPalettes[] = {
+static const struct palette video_palettes[] = {
 	{"CGA 1", 		// CGA black, cyan, magenta, white (Sopwith's default color scheme)
 		{{0, 0, 0}, {0, 255, 255}, {255, 0, 255}, {255, 255, 255}}},
 	{"CGA 2", 		// CGA black, red, green, yellow
@@ -66,13 +62,13 @@ VideoPalette VideoPalettes[] = {
 	{"Gas Plasma",
 		{{0x7d, 0x1b, 0x02}, {0xd3, 0x41, 0x00},
 		 {0xa8, 0x2e, 0x01}, {0xfe, 0x54, 0x00}}},
+	// Palette from swgrapha.c in the original source, the colors
+	// that would have appeared in the Atari port.
+	{"Atari",
+		{{0x00, 0x00, 0x00}, {0x00, 0x77, 0xff},
+		 {0xff, 0x00, 0x00}, {0xff, 0xff, 0xff}}},
 };
 
-bool vid_fullscreen = false;
-
-extern unsigned char *vid_vram;
-extern unsigned int vid_pitch;
-extern int gamenum;
 extern bool isNetworkGame(void);
 
 int keybindings[NUM_KEYS] = {
@@ -90,11 +86,14 @@ int keybindings[NUM_KEYS] = {
 	SDL_SCANCODE_S,       // KEY_SOUND
 };
 
+bool vid_fullscreen = false;
 static int ctrlbreak = 0;
-static bool initted = 0;
+static bool initted = false;
 static SDL_Window *window = NULL;
 static uint32_t pixel_format;
 static SDL_Renderer *renderer;
+static SDL_Keysym input_buffer[INPUT_BUFFER_LEN];
+static int input_buffer_head = 0, input_buffer_tail = 0;
 
 // Maximum number of pixels to use for intermediate scale buffer.
 static int max_scaling_buffer_pixels = 16000000;
@@ -110,17 +109,16 @@ static SDL_Surface *argbbuffer = NULL;
 static SDL_Texture *texture = NULL;
 static SDL_Texture *texture_upscaled = NULL;
 
-
 // convert a sopsym_t into a surface
 #define ICON_SCALE 4
-static SDL_Surface *surface_from_sopsym(sopsym_t *sym)
+static SDL_Surface *SurfaceFromSopsym(sopsym_t *sym)
 {
-	SDL_Color *pal = VideoPalettes[0].color;
+	const SDL_Color *pal = video_palettes[0].color;
 	SDL_Surface *surface = SDL_CreateRGBSurface(
 		0, sym->w * ICON_SCALE, sym->h * ICON_SCALE, 32,
 		0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
 
-	unsigned char *src;
+	uint8_t *src;
 	uint32_t *dst;
 	int x, y, sx, sy;
 
@@ -138,7 +136,7 @@ static SDL_Surface *surface_from_sopsym(sopsym_t *sym)
 		dst = (uint32_t *) ((uint8_t *) surface->pixels +
 		                    y * surface->pitch);
 		for (x = 0; x < surface->w; ++x) {
-			SDL_Color *p;
+			const SDL_Color *p;
 			sx = x / ICON_SCALE;
 			if (src[sx] == 0) {
 				dst[x] = 0;
@@ -155,8 +153,6 @@ static SDL_Surface *surface_from_sopsym(sopsym_t *sym)
 	return surface;
 }
 
-// 2x scale
-
 void Vid_Update(void)
 {
 	static SDL_Rect blit_rect = { 0, 0, SCR_WDTH, SCR_HGHT };
@@ -192,7 +188,7 @@ void Vid_Update(void)
 	SDL_LockSurface(screenbuf);
 }
 
-static bool is_special_day(void)
+static bool IsSpecialDay(void)
 {
 	time_t now = time(NULL);
 	struct tm *t = localtime(&now);
@@ -211,18 +207,18 @@ static bool is_special_day(void)
 	 || (t->tm_mon == 3 && t->tm_mday == 24);
 }
 
-static void set_icon(void)
+static void SetIcon(void)
 {
 	SDL_Surface *icon;
 	sopsym_t *sym;
 
-	if (is_special_day()) {
-		sym = &symbol_plane[0]->sym[4];
+	if (IsSpecialDay()) {
+		sym = &symbol_plane[0].sym[4];
 	} else {
-		sym = &symbol_plane[0]->sym[0];
+		sym = &symbol_plane[0].sym[0];
 	}
 
-	icon = surface_from_sopsym(sym);
+	icon = SurfaceFromSopsym(sym);
 	if (icon == NULL) {
 		return;
 	}
@@ -242,8 +238,8 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 
 	// Query renderer and limit to maximum texture dimensions of hardware:
 	if (SDL_GetRendererInfo(renderer, &rinfo) != 0) {
-		error_exit("CreateUpscaledTexture: SDL_GetRendererInfo() "
-		           "call failed: %s", SDL_GetError());
+		ErrorExit("CreateUpscaledTexture: SDL_GetRendererInfo() "
+		          "call failed: %s", SDL_GetError());
 	}
 
 	while (*w_upscale * SCR_WDTH > rinfo.max_texture_width) {
@@ -255,10 +251,10 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 
 	if ((*w_upscale < 1 && rinfo.max_texture_width > 0)
 	 || (*h_upscale < 1 && rinfo.max_texture_height > 0)) {
-		error_exit("CreateUpscaledTexture: Can't create a "
-		           "texture big enough for the whole screen! "
-		           "Maximum texture size %dx%d",
-		           rinfo.max_texture_width, rinfo.max_texture_height);
+		ErrorExit("CreateUpscaledTexture: Can't create a "
+		          "texture big enough for the whole screen! "
+		          "Maximum texture size %dx%d",
+		          rinfo.max_texture_width, rinfo.max_texture_height);
 	}
 
 	// We limit the amount of texture memory used for the intermediate buffer,
@@ -267,10 +263,10 @@ static void LimitTextureSize(int *w_upscale, int *h_upscale)
 	// huge textures, so the user can use this to reduce the maximum texture
 	// size if desired.
 	if (max_scaling_buffer_pixels < SCR_WDTH * SCR_HGHT) {
-		error_exit("CreateUpscaledTexture: max_scaling_buffer_"
-		           "pixels too small to create a texture buffer:"
-		           " %d < %d", max_scaling_buffer_pixels,
-		           SCR_WDTH * SCR_HGHT);
+		ErrorExit("CreateUpscaledTexture: max_scaling_buffer_"
+		          "pixels too small to create a texture buffer:"
+		          " %d < %d", max_scaling_buffer_pixels,
+		          SCR_WDTH * SCR_HGHT);
 	}
 
 	while (*w_upscale * *h_upscale * SCR_WDTH * SCR_HGHT
@@ -302,7 +298,7 @@ static void CreateUpscaledTexture(int force)
 	// real world pixels, which are not necessarily equivalent to the
 	// screen's window size (because of highdpi).
 	if (SDL_GetRendererOutputSize(renderer, &w, &h) != 0) {
-		error_exit("failed to get renderer size: %s", SDL_GetError());
+		ErrorExit("failed to get renderer size: %s", SDL_GetError());
 	}
 
 	// When the screen or window dimensions do not match the aspect ratio
@@ -392,8 +388,8 @@ static void Vid_SetMode(void)
 	int bpp;
 
 	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
-		error_exit("Unable to initialize video subsystem: %s",
-		           SDL_GetError());
+		ErrorExit("Unable to initialize video subsystem: %s",
+		          SDL_GetError());
 	}
 	srand(time(NULL));
 
@@ -409,18 +405,18 @@ static void Vid_SetMode(void)
 	                          SDL_WINDOWPOS_CENTERED, w, h, flags);
 
 	if (window == NULL) {
-		error_exit("Failed to open SDL window: %s", SDL_GetError());
+		ErrorExit("Failed to open SDL window: %s", SDL_GetError());
 	}
 
-        SDL_StopTextInput();
+	SDL_StopTextInput();
 
-        pixel_format = SDL_GetWindowPixelFormat(window);
+	pixel_format = SDL_GetWindowPixelFormat(window);
 
 	for (n = 0; n < NUM_KEYS; ++n) {
 		keysdown[n] = 0;
 	}
 
-	set_icon();
+	SetIcon();
 	SDL_ShowCursor(0);
 
 	renderer_flags = SDL_RENDERER_PRESENTVSYNC;
@@ -484,7 +480,7 @@ void Vid_Shutdown(void)
 
 	SDL_FreeSurface(screenbuf);
 
-	initted = 0;
+	initted = false;
 }
 
 void Vid_Init(void)
@@ -501,10 +497,11 @@ void Vid_Init(void)
 	                                 0, 0, 0, 0);
 	vid_vram = screenbuf->pixels;
 	vid_pitch = screenbuf->pitch;
-	SDL_SetPaletteColors(screenbuf->format->palette, cga_pal, 0,
-	                     sizeof(cga_pal) / sizeof(*cga_pal));
+	SDL_SetPaletteColors(screenbuf->format->palette,
+	                     video_palettes[0].color, 0,
+	                     arrlen(video_palettes[0].color));
 
-	initted = 1;
+	initted = true;
 
 	atexit(Vid_Shutdown);
 
@@ -521,36 +518,28 @@ void Vid_Reset(void)
 	Vid_SetMode();
 
 	// need to redraw buffer to screen
-
 	Vid_Update();
 }
 
 void Vid_SetVideoPalette(int palette)
 {
-	cga_pal[0] = VideoPalettes[palette].color[0];
-	cga_pal[1] = VideoPalettes[palette].color[1];
-	cga_pal[2] = VideoPalettes[palette].color[2];
-	cga_pal[3] = VideoPalettes[palette].color[3];
-	SDL_SetPaletteColors(screenbuf->format->palette, cga_pal, 0, sizeof(cga_pal) / sizeof(*cga_pal));
+	SDL_SetPaletteColors(screenbuf->format->palette,
+	                     video_palettes[palette].color, 0,
+	                     arrlen(video_palettes[palette].color));
 	Vid_Update();
 }
 
 const char* Vid_GetVideoPaletteName(int palette)
 {
-        return VideoPalettes[palette].name;
+	return video_palettes[palette].name;
 }
 
 int Vid_GetNumVideoPalettes(void)
 {
-    int numPalettes = sizeof(VideoPalettes) / sizeof(VideoPalettes[0]);
-	return numPalettes;
+	return arrlen(video_palettes);
 }
 
-#define INPUT_BUFFER_LEN 32
-static SDL_Keysym input_buffer[INPUT_BUFFER_LEN];
-static int input_buffer_head = 0, input_buffer_tail = 0;
-
-static void input_buffer_push(SDL_Keysym c)
+static void InputBufferPush(SDL_Keysym c)
 {
 	int tail_next = (input_buffer_tail + 1) % INPUT_BUFFER_LEN;
 	if (tail_next == input_buffer_head) {
@@ -560,7 +549,7 @@ static void input_buffer_push(SDL_Keysym c)
 	input_buffer_tail = tail_next;
 }
 
-static SDL_Keysym input_buffer_pop(void)
+static SDL_Keysym InputBufferPop(void)
 {
 	SDL_Keysym result;
 
@@ -575,7 +564,7 @@ static SDL_Keysym input_buffer_pop(void)
 	return result;
 }
 
-static sopkey_t translate_scancode(int sdl_scancode)
+static enum gamekey TranslateScancode(int sdl_scancode)
 {
 	int i;
 
@@ -641,48 +630,59 @@ static void CtrlKeyPress(SDL_Keycode k)
 	}
 }
 
-static void getevents(void)
+static void KeyDown(SDL_KeyboardEvent *event)
+{
+	enum gamekey translated;
+
+	if (CtrlDown()) {
+		CtrlKeyPress(event->keysym.sym);
+		return;
+	} else if (AltDown() && (event->keysym.sym == SDLK_RETURN
+	                      || event->keysym.sym == SDLK_KP_ENTER)) {
+#ifndef NO_FULLSCREEN
+		vid_fullscreen = !vid_fullscreen;
+		Vid_Reset();
+#endif
+		return;
+	} else if (event->keysym.sym == SDLK_KP_ENTER
+	        && SDL_IsTextInputActive()) {
+		SDL_Keysym fake = {0};
+		fake.sym = '\n';
+		fake.scancode = SDL_SCANCODE_UNKNOWN;
+		InputBufferPush(fake);
+		return;
+	} else if (!SDL_IsTextInputActive() || IsSpecialKey(&event->keysym)) {
+		InputBufferPush(event->keysym);
+	}
+
+	translated = TranslateScancode(event->keysym.scancode);
+	if (translated != KEY_UNKNOWN) {
+		keysdown[translated] |= 3;
+	}
+}
+
+static void KeyUp(SDL_KeyboardEvent *event)
+{
+	enum gamekey translated = TranslateScancode(event->keysym.scancode);
+	if (translated != KEY_UNKNOWN) {
+		keysdown[translated] &= ~1;
+	}
+}
+
+static void GetEvents(void)
 {
 	SDL_Event event;
-	int need_redraw = 0;
+	bool need_redraw = false;
 	int i;
-	sopkey_t translated;
 
 	while (SDL_PollEvent(&event)) {
 		switch (event.type) {
 		case SDL_KEYDOWN:
-			if (CtrlDown()) {
-				CtrlKeyPress(event.key.keysym.sym);
-			} else if (AltDown() && (event.key.keysym.sym == SDLK_RETURN
-			                      || event.key.keysym.sym == SDLK_KP_ENTER)) {
-#ifndef NO_FULLSCREEN
-				vid_fullscreen = !vid_fullscreen;
-				Vid_Reset();
-				continue;
-#endif
-			} else if (event.key.keysym.sym == SDLK_KP_ENTER && SDL_IsTextInputActive()) {
-				SDL_Keysym fake = {0};
-				fake.sym = '\n';
-				fake.scancode = SDL_SCANCODE_UNKNOWN;
-				input_buffer_push(fake);
-				continue;
-			} else if (!SDL_IsTextInputActive()
-			 || IsSpecialKey(&event.key.keysym)) {
-				input_buffer_push(event.key.keysym);
-			}
-			translated = translate_scancode(
-				event.key.keysym.scancode);
-			if (translated != KEY_UNKNOWN) {
-				keysdown[translated] |= 3;
-			}
+			KeyDown(&event.key);
 			break;
 
 		case SDL_KEYUP:
-			translated = translate_scancode(
-				event.key.keysym.scancode);
-			if (translated != KEY_UNKNOWN) {
-				keysdown[translated] &= ~1;
-			}
+			KeyUp(&event.key);
 			break;
 
 		case SDL_TEXTINPUT:
@@ -694,7 +694,7 @@ static void getevents(void)
 				}
 				fake.sym = c;
 				fake.scancode = SDL_SCANCODE_UNKNOWN;
-				input_buffer_push(fake);
+				InputBufferPush(fake);
 			}
 			break;
 
@@ -708,7 +708,7 @@ static void getevents(void)
 				// When we get one of these events, we redraw
 				// the screen immediately, as we may be in a
 				// menu waiting for a keypress.
-				need_redraw = 1;
+				need_redraw = true;
 				break;
 			}
 		}
@@ -732,16 +732,16 @@ void Vid_StopTextInput(void)
 int Vid_GetKey(void)
 {
 	SDL_Keysym k;
-	getevents();
-	k = input_buffer_pop();
+	GetEvents();
+	k = InputBufferPop();
 	return k.scancode;
 }
 
 int Vid_GetChar(void)
 {
 	int result;
-	getevents();
-	result = input_buffer_pop().sym;
+	GetEvents();
+	result = InputBufferPop().sym;
 	if (result == '\r') {
 		result = '\n';
 	}
@@ -750,7 +750,7 @@ int Vid_GetChar(void)
 
 bool Vid_GetCtrlBreak(void)
 {
-	getevents();
+	GetEvents();
 	return ctrlbreak;
 }
 
@@ -783,7 +783,7 @@ int isatty(int fd)
 }
 #endif
 
-void error_exit(char *s, ...)
+void ErrorExit(char *s, ...)
 {
 	static char buf[128];
 	va_list args;
