@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 //        swauto   -      SW control of computer player
@@ -82,27 +81,47 @@ static int shoot(OBJECTS *obt)
 	return 0;
 }
 
+static bool WithinHomeRange(OBJECTS *ob, int x, int y)
+{
+	const original_ob_t *orig_ob = ob->ob_original_ob;
 
-static bool is_target(OBJECTS *ob)
+	return abs(x - orig_ob->x) < HOME
+	    && abs(y - ob->ob_orig_y) < HOME;
+}
+
+static bool IsTarget(OBJECTS *ob)
 {
 	return ob->ob_type == TARGET || ob->ob_type == OX;
 }
 
-static bool tstcrash2(OBJECTS *ob, int x, int y, int alt)
+static bool tstcrash2(OBJECTS *ob, int x, int y, int alt, int dy)
 {
 	OBJECTS *obt;
-	int xl, xr, yt;
+	int xl, xr, yt, lookahead = 3;
 
 	if (alt > 50) {
 		return false;
 	}
 
-	// If we're going to get too low, don't aim downwards.
+	// If we're heading home and this will be the last step here, we
+	// don't need to look three tics ahead. Otherwise at high speeds the
+	// autopilot can end up endlessly circling the runway, afraid any
+	// downward movement at all will lead to a crash.
+	if (ob->ob_home && WithinHomeRange(ob, x, y)) {
+		lookahead = 1;
+	}
+
+	// If we're going to get too low, don't aim downwards. This is
+	// expressed in terms of the speed we will be heading towards
+	// the ground; if we are likely to crash within three tics, we
+	// should try to pull away. This approach is independent of the
+	// speed we are travelling at, so it works at the higher levels
+	// where the plane travels very fast.
 	// The behavior here has been fixed compared to the original
 	// Author's Edition release. It used to be that planes would
 	// sometimes try to take off by flying straight upwards without
 	// full throttle.
-	if (alt < 22 && ob->ob_dy < 0) {
+	if (alt + dy * lookahead / 256 < 8) {
 		return true;
 	}
 
@@ -123,9 +142,8 @@ static bool tstcrash2(OBJECTS *ob, int x, int y, int alt)
 		obt = obt->ob_xprev;
 	}
 
-
 	for (; obt->ob_xnext != NULL; obt = obt->ob_xnext) {
-		if (!is_target(obt) || obt->ob_x < xl) {
+		if (!IsTarget(obt) || obt->ob_x < xl) {
 			continue;
 		}
 		if (obt->ob_x > xr) {
@@ -144,13 +162,13 @@ int aim(OBJECTS *ob, int ax, int ay, OBJECTS *obt, bool longway)
 	int r, rmin, i, n=0;
 	int x, y, dx, dy, nx, ny;
 	int nangle, nspeed;
-	static int cflaps[3] = { 0, -1, 1 };
+	static const int cflaps[3] = { 0, -1, 1 };
 	static int crange[3], ccrash[3], calt[3];
 
 	correction = false;
 
-	if ((ob->ob_state == STALLED || ob->ob_state == WOUNDSTALL)
-	    && ob->ob_angle != (3 * ANGLES / 4)) {
+	if (PlaneIsStalled(ob->ob_state)
+	 && ob->ob_angle != (3 * ANGLES / 4)) {
 		ob->ob_flaps = -1;
 		ob->ob_accel = MAX_THROTTLE;
 		return 0;
@@ -223,12 +241,11 @@ int aim(OBJECTS *ob, int ax, int ay, OBJECTS *obt, bool longway)
 		movexy(&obs, &nx, &ny);
 		crange[i] = range(nx, ny, ax, ay);
 		calt[i] = ny - currgame->gm_ground[nx + 8];
-		ccrash[i] = tstcrash2(ob, nx, ny, calt[i]);
+		ccrash[i] = tstcrash2(ob, nx, ny, calt[i], nspeed * SIN(nangle));
 
 		copyobj(&obs, ob);
 	}
 
-
 	if (obt) {
 		i = shoot(obt);
 
@@ -299,21 +316,14 @@ int aim(OBJECTS *ob, int ax, int ay, OBJECTS *obt, bool longway)
 	return 0;
 }
 
-
-
 int gohome(OBJECTS *ob)
 {
-	const original_ob_t *orig_ob;
-
 	if (ob->ob_athome) {
 		return 0;
 	}
 
-	orig_ob = ob->ob_original_ob;
-
 	courseadj = ((countmove & 0x001F) < 16) << 4;
-	if (abs(ob->ob_x - orig_ob->x) < HOME
-	 && abs(ob->ob_y - ob->ob_orig_y) < HOME) {
+	if (WithinHomeRange(ob, ob->ob_x, ob->ob_y)) {
 		if (plyrplane) {
 			initplyr(ob);
 			initdisp(true);
@@ -330,13 +340,11 @@ int gohome(OBJECTS *ob)
 	if (ob->ob_state == WOUNDED && (countmove & 1)) {
 		return 0;
 	} else {
-		return aim(ob, orig_ob->x, ob->ob_orig_y, NULL, false);
+		return aim(ob, ob->ob_original_ob->x, ob->ob_orig_y,
+		           NULL, false);
 	}
 }
 
-
-
-
 static void cruise(OBJECTS *ob)
 {
 	int orgx;
@@ -362,7 +370,6 @@ static void attack(OBJECTS *obp, OBJECTS *ob)
 	}
 }
 
-
 void swauto(OBJECTS *ob)
 {
 	if (ob->ob_target != NULL) {
@@ -374,7 +381,6 @@ void swauto(OBJECTS *ob)
 	ob->ob_target = NULL;
 }
 
-
 int range(int x, int y, int ax, int ay)
 {
 	int dx, dy;
