@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 //        swcollsn -      SW collision resolution
@@ -19,11 +18,12 @@
 #include "swinit.h"
 #include "swmain.h"
 #include "swmove.h"
+#include "swobject.h"
 #include "swsound.h"
 #include "swsplat.h"
 
 static void tstcrash(OBJECTS *obp);
-static void blast_target(OBJECTS *ob, obtype_t type);
+static void BlastTarget(OBJECTS *ob, obtype_t type);
 
 static OBJECTS *killed[MAX_OBJS*2], *killer[MAX_OBJS*2];
 static int killptr;
@@ -34,7 +34,6 @@ static OBJECTS *collsno[MAX_PLYR];
 static int collptr;
 static int collxadj, collyadj;
 
-
 //#define COLL_DEBUG
 
 static void colltest(OBJECTS * ob1, OBJECTS * ob2)
@@ -42,7 +41,7 @@ static void colltest(OBJECTS * ob1, OBJECTS * ob2)
 	int x, y;
 	int x1, y1, x2, y2;
 	int w, h;
-	unsigned char *data1, *data2;
+	uint8_t *data1, *data2;
 
 	if ((ob1->ob_type == PLANE && ob1->ob_state >= FINISHED)
 	 || (ob2->ob_type == PLANE && ob2->ob_state >= FINISHED)
@@ -117,7 +116,7 @@ static void colltest(OBJECTS * ob1, OBJECTS * ob2)
 	data2 = ob2->ob_newsym->data + ob2->ob_newsym->w * y2 + x2;
 
 	for (y=0; y<h; ++y) {
-		unsigned char *d1 = data1, *d2 = data2;
+		uint8_t *d1 = data1, *d2 = data2;
 
 		for (x=0; x<w; ++x) {
 			if (*d1 && *d2) {
@@ -143,11 +142,9 @@ static void colltest(OBJECTS * ob1, OBJECTS * ob2)
 	}
 }
 
-
-static OBJECTS *
-get_score_obj(OBJECTS *ob, int *reverse)
-/* Determine the object that receives a score if 'ob' is destriyed, and whether the score
-** should be subtracted rather than added (iff reverse nonzero) */
+/* Determine the object that receives a score if 'ob' is destriyed, and whether
+ * the score should be subtracted rather than added (iff reverse nonzero) */
+static OBJECTS *GetScoreObject(OBJECTS *ob, int *reverse)
 {
 	OBJECTS *retval;
 	*reverse = 0;
@@ -168,7 +165,7 @@ static void scoretarg(OBJECTS *obp, int score)
 	OBJECTS *ob;
 	int reverse_score;
 
-	ob = get_score_obj(obp, &reverse_score);
+	ob = GetScoreObject(obp, &reverse_score);
 	if (reverse_score) {
 		ob->ob_score.score -= score;
 	} else {
@@ -176,7 +173,6 @@ static void scoretarg(OBJECTS *obp, int score)
 	}
 }
 
-
 static bool scorepenalty(obtype_t ttype, OBJECTS * ob, int score)
 {
 	OBJECTS *obt;
@@ -195,10 +191,7 @@ static bool scorepenalty(obtype_t ttype, OBJECTS * ob, int score)
 	return false;
 }
 
-
-
-
-static int crtdepth[8] = { 1, 2, 2, 3, 3, 2, 2, 1 };
+static const int crtdepth[8] = { 1, 2, 2, 3, 3, 2, 2, 1 };
 
 static void crater(OBJECTS * ob)
 {
@@ -224,11 +217,9 @@ static void crater(OBJECTS * ob)
 
 /* Determine whether the parameter is a shot and hasn't moved very much yet;
 ** Used to avoid having planes hitting themselves */
-
-static int
-is_young_shot(OBJECTS *ob)
+static bool IsYoungShot(OBJECTS *ob)
 {
-	return (ob && ob->ob_type == SHOT && ob->ob_life >= BULLIFE - 1);
+	return ob && ob->ob_type == SHOT && ob->ob_life >= BULLIFE - 1;
 }
 
 static void swkill(OBJECTS * ob1, OBJECTS * ob2)
@@ -261,7 +252,7 @@ static void swkill(OBJECTS * ob1, OBJECTS * ob2)
 		/* cr 2005-04-28: Don't stop the shot if it just
 		 * launched from its presumed originator */
 
-		if (!(obt && obt->ob_type == PLANE && is_young_shot(ob))) {
+		if (!(obt && obt->ob_type == PLANE && IsYoungShot(ob))) {
 			ob->ob_life = 1;
 		}
 		return;
@@ -298,8 +289,8 @@ static void swkill(OBJECTS * ob1, OBJECTS * ob2)
 		ob->ob_onmap = false;
 		initexpl(ob, 0);
 
-		blast_target(ob, ttype);
-		scoretarg(ob, ob->ob_orient == 2 ? 200 : 100);
+		BlastTarget(ob, ttype);
+		scoretarg(ob, ob->ob_orient == TARGET_OIL_TANK ? 200 : 100);
 
 		--numtarg[ob->ob_clr - 1];
 		if (numtarg[ob->ob_clr - 1] <= 0
@@ -314,7 +305,7 @@ static void swkill(OBJECTS * ob1, OBJECTS * ob2)
 
 		/* cr 2005-04-28: Avoid having planes hit themselves */
 
-		if (is_young_shot(obt)) {
+		if (IsYoungShot(obt)) {
 			return;	
 		}
 
@@ -441,7 +432,6 @@ static void swkill(OBJECTS * ob1, OBJECTS * ob2)
 	}
 }
 
-
 void swcollsn(void)
 {
 	OBJECTS *ob, *obp, **obkd, **obkr;
@@ -527,13 +517,10 @@ static void tstcrash(OBJECTS *obp)
 	}
 }
 
-
-/* Determine valour factor */
-
-static int compute_valour(OBJECTS *ob)
+static int ComputeValour(OBJECTS *ob)
 {
 	int reverse;
-	OBJECTS *so = get_score_obj(ob, &reverse);
+	OBJECTS *so = GetScoreObject(ob, &reverse);
 	int x_home;
 	int distance;
 	int valour = 0;
@@ -563,8 +550,7 @@ static int compute_valour(OBJECTS *ob)
 		valour++;
 	}
 
-	if (so->ob_state == WOUNDED
-	    || so->ob_state == WOUNDSTALL) {
+	if (PlaneIsWounded(so->ob_state)) {
 		valour = (valour + 1) * 3;
 	} else {
 		valour *= 2;
@@ -573,10 +559,10 @@ static int compute_valour(OBJECTS *ob)
 	return valour;
 }
 
-static void blast_target(OBJECTS *ob, obtype_t type)
+static void BlastTarget(OBJECTS *ob, obtype_t type)
 {
 	int reverse;
-	OBJECTS *so = get_score_obj(ob, &reverse);
+	OBJECTS *so = GetScoreObject(ob, &reverse);
 
 	if (reverse) {
 		return;
@@ -584,7 +570,7 @@ static void blast_target(OBJECTS *ob, obtype_t type)
 
 	if (type == BOMB || type == SHOT || type == MISSILE || type == PLANE) {
 		so->ob_flightscore.killscore += 4;
-		so->ob_flightscore.valour += 3 * compute_valour (ob);
+		so->ob_flightscore.valour += 3 * ComputeValour (ob);
 	}
 }
 
@@ -596,14 +582,14 @@ void scorepln(OBJECTS * ob, obtype_t type)
 
 	if (type == BOMB || type == SHOT || type == MISSILE || type == PLANE) {
 		int reverse;
-		OBJECTS *scobj = get_score_obj(ob, &reverse);
+		OBJECTS *scobj = GetScoreObject(ob, &reverse);
 		if (!reverse) {
 			if (had_taken_off) {
 				if (type != PLANE) {
 					scobj->ob_flightscore.planekills++;
 				}
 				scobj->ob_flightscore.valour +=
-					4 * (2 + compute_valour (ob));
+					4 * (2 + ComputeValour (ob));
 			}
 
 			scobj->ob_flightscore.killscore += 3;
