@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 //        swinit   -      SW initialization
@@ -44,7 +43,7 @@ static bool have_savescore = false;
 static score_t savescore;		/* save players score on restart  */
 int starting_level = 0;
 
-static char helptxt[] =
+static const char *helptxt =
 "\n"
 PACKAGE_STRING "\n"
 "Copyright (C) 1984, 1985, 1987 BMB Compuscience\n"
@@ -83,7 +82,6 @@ static void initobjs(void)
 	objfree = NULL;
 }
 
-
 void initgrnd(void)
 {
 	free(ground);
@@ -106,7 +104,6 @@ void initdisp(bool reset)
 	}
 }
 
-
 //
 // object creation
 //
@@ -131,7 +128,7 @@ static void initflightscore(flight_score_t *score)
 #define VALOUR_PRELIMIT 175
 #define VALOUR_LIMIT 250
 
-static bool have_medal(int *medals, int medals_nr, int medal_id)
+static bool HaveMedal(int *medals, int medals_nr, int medal_id)
 {
 	int i;
 
@@ -144,25 +141,25 @@ static bool have_medal(int *medals, int medals_nr, int medal_id)
 	return false;
 }
 
-static void give_medal(OBJECTS *ob, int medal_id)
+static void GiveMedal(OBJECTS *ob, int medal_id)
 {
 	score_t *sc = &ob->ob_score;
-	if (!have_medal(sc->medals, sc->medals_nr, medal_id)) {
+	if (!HaveMedal(sc->medals, sc->medals_nr, medal_id)) {
 		assert(sc->medals_nr < 3);
 		sc->medals[sc->medals_nr++] = medal_id;
 	}
 }
 
-static void give_ribbon(OBJECTS *ob, int ribbon_id)
+static void GiveRibbon(OBJECTS *ob, int ribbon_id)
 {
 	score_t *sc = &ob->ob_score;
-	if (!have_medal(sc->ribbons, sc->ribbons_nr, ribbon_id)) {
+	if (!HaveMedal(sc->ribbons, sc->ribbons_nr, ribbon_id)) {
 		assert(sc->ribbons_nr < 6);
 		sc->ribbons[sc->ribbons_nr++] = ribbon_id;
 	}
 }
 
-static void get_awards(OBJECTS *ob)
+static void GetAwards(OBJECTS *ob)
 {
 	flight_score_t *fsc = &ob->ob_flightscore;
 	score_t *sc = &ob->ob_score;
@@ -170,7 +167,7 @@ static void get_awards(OBJECTS *ob)
 	// We only award purple heart if wounded in combat (=hit by bullet),
 	// not from a bird strike or flying into an ox.
 	if (fsc->combatwound) {
-		give_medal(ob, MEDAL_PURPLEHEART);
+		GiveMedal(ob, MEDAL_PURPLEHEART);
 	}
 
 	// We count up the number of planes shot down, but they only count if
@@ -178,10 +175,10 @@ static void get_awards(OBJECTS *ob)
 	sc->planekills += fsc->planekills;
 	//printf("planes %d %d\n", sc->planekills, fsc->planekills);
 	if (sc->planekills >= 5) {
-		give_ribbon(ob, RIBBON_ACE);
+		GiveRibbon(ob, RIBBON_ACE);
 	}
 	if (sc->planekills >= 25) {
-		give_ribbon(ob, RIBBON_TOPACE);
+		GiveRibbon(ob, RIBBON_TOPACE);
 	}
 
 	// We count the number of landings during which we did a decent amount
@@ -191,16 +188,16 @@ static void get_awards(OBJECTS *ob)
 	}
 	//printf("kills %d landings %d\n", fsc->killscore, sc->landings);
 	if (sc->landings >= 3) {
-		give_ribbon(ob, RIBBON_SERVICE);
+		GiveRibbon(ob, RIBBON_SERVICE);
 	}
 
 	// If a huge number of damage is done, we give a medal (first time) or
 	// ribbon (second time)
 	if (fsc->killscore >= COMPETENCE_KILLSCORE) {
-		if (!have_medal(sc->medals, sc->medals_nr, MEDAL_COMPETENCE)) {
-			give_medal(ob, MEDAL_COMPETENCE);
+		if (!HaveMedal(sc->medals, sc->medals_nr, MEDAL_COMPETENCE)) {
+			GiveMedal(ob, MEDAL_COMPETENCE);
 		} else {
-			give_ribbon(ob, RIBBON_COMPETENCE2);
+			GiveRibbon(ob, RIBBON_COMPETENCE2);
 		}
 	}
 
@@ -209,19 +206,19 @@ static void get_awards(OBJECTS *ob)
 	sc->valour += fsc->valour;
 	//printf("valour %d %d\n", sc->valour, fsc->valour);
 	if (sc->valour >= VALOUR_PRELIMIT) {
-		give_ribbon(ob, RIBBON_PREVALOUR);
+		GiveRibbon(ob, RIBBON_PREVALOUR);
 	}
 	if (sc->valour >= VALOUR_LIMIT) {
-		give_medal(ob, MEDAL_VALOUR);
+		GiveMedal(ob, MEDAL_VALOUR);
 	}
 }
 
-static void get_endlevel(OBJECTS *ob)
+static void GetEndLevel(OBJECTS *ob)
 {
-	get_awards(ob);
+	GetAwards(ob);
 
 	if (ob->ob_crashcnt == 0) {
-		give_ribbon(ob, RIBBON_PERFECT);
+		GiveRibbon(ob, RIBBON_PERFECT);
 	}
 }
 
@@ -244,7 +241,7 @@ OBJECTS *initpln(OBJECTS * obp)
 
 	if (obp && ob->ob_state != CRASHED && !ob->ob_athome) {
 		/* Just returned home */
-		get_awards(ob);
+		GetAwards(ob);
 	}
 
 	initflightscore(&ob->ob_flightscore);
@@ -272,7 +269,7 @@ OBJECTS *initpln(OBJECTS * obp)
 	ob->ob_target = ob->ob_missiletarget = NULL;
 	ob->ob_firing = ob->ob_mfiring = NULL;
 	ob->ob_bombing = ob->ob_bfiring = ob->ob_home = false;
-	ob->ob_newsym = &symbol_plane[0]->sym[ob->ob_orient ? 4 : 0];
+	ob->ob_newsym = &symbol_plane[0].sym[ob->ob_orient ? 4 : 0];
 	ob->ob_athome = true;
 	ob->ob_onmap = true;
 	ob->ob_flightscore.combatwound = false;
@@ -300,8 +297,6 @@ OBJECTS *initpln(OBJECTS * obp)
 	return ob;
 }
 
-
-
 // player
 
 void initplyr(OBJECTS * obp)
@@ -348,7 +343,6 @@ void initcomp(OBJECTS * obp)
 	}
 }
 
-
 static int isrange(int x, int y, int ax, int ay)
 {
 	int dx, dy, t;
@@ -472,7 +466,7 @@ void initbomb(OBJECTS *obo)
 	ob->ob_life = BOMBLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_bomb[0]->sym[0];
+	ob->ob_newsym = &symbol_bomb[0].sym[0];
 	ob->ob_drawf = dispbomb;
 	ob->ob_movef = movebomb;
 
@@ -515,7 +509,7 @@ void initmiss(OBJECTS *obo)
 	ob->ob_life = MISSLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_missile[0]->sym[0];
+	ob->ob_newsym = &symbol_missile[0].sym[0];
 	ob->ob_drawf = NULL;
 	ob->ob_movef = movemiss;
 	ob->ob_missiletarget = obo->ob_mfiring;
@@ -526,7 +520,6 @@ void initmiss(OBJECTS *obo)
 
 }
 
-
 // starburst
 
 void initburst(OBJECTS *obo)
@@ -569,7 +562,7 @@ void initburst(OBJECTS *obo)
 	ob->ob_life = BURSTLIFE;
 	ob->ob_owner = obo;
 	ob->ob_clr = obo->ob_clr;
-	ob->ob_newsym = &symbol_burst[0]->sym[0];
+	ob->ob_newsym = &symbol_burst[0].sym[0];
 	ob->ob_drawf = NULL;
 	ob->ob_movef = moveburst;
 
@@ -646,7 +639,7 @@ static OBJECTS *inittarget(const original_ob_t *orig_ob)
 			break;
 	}
 	ob->ob_clr = ob->ob_owner->ob_clr;
-	ob->ob_newsym = &symbol_targets[0]->sym[0];
+	ob->ob_newsym = &symbol_targets[0].sym[0];
 	ob->ob_drawf = disptarg;
 	ob->ob_movef = movetarg;
 	ob->ob_onmap = true;
@@ -654,6 +647,29 @@ static OBJECTS *inittarget(const original_ob_t *orig_ob)
 	return ob;
 }
 
+// Number of explosion fragments is calculated based on the number of pixels
+// in the "destroyed target" sprite:
+static int TargetExplosionSize(int target_type)
+{
+	symset_t *ss = &symbol_target_hit[target_type];
+	sopsym_t *s = &ss->sym[0];
+	int i, num_pixels;
+
+	for (i = 0, num_pixels = 0; i < s->w * s->h; i++) {
+		if (s->data[i] != 0) {
+			++num_pixels;
+		}
+	}
+
+	if (num_pixels > 64) {
+		return 64;
+	} else if (num_pixels < 10) {
+		return 10;
+	} else {
+		return num_pixels;
+	}
+}
+
 // explosion
 
 void initexpl(OBJECTS *obo, int small)
@@ -673,7 +689,12 @@ void initexpl(OBJECTS *obo, int small)
 	oboclr = obo->ob_clr;
 
 	obotype = obo->ob_type;
-	if (obotype == TARGET && obo->ob_orient == 2) {
+
+	if (obotype == TARGET
+	 && (obo->ob_orient == TARGET_OIL_TANK
+	  || obo->ob_orient == TARGET_TANKER_TRUCK
+	  || obo->ob_orient == TARGET_CUSTOM5
+	  || obo->ob_orient == TARGET_CUSTOM_PASSIVE5)) {
 		ic = 1;
 		// sdh: Oil tank explosions were changed in Sopwith "Network
 		// Edition" to be much less intense; we make this change
@@ -686,8 +707,19 @@ void initexpl(OBJECTS *obo, int small)
 			speed = gminspeed;
 		}
 	} else {
-		ic = small ? 6 : 2;
 		speed = gminspeed >> ((explseed & 7) != 7);
+		if (small) {
+			ic = 6;
+		} else if (obotype == TARGET) {
+			// For targets we calculate the number of fragments
+			// based on the number of pixels in the destroyed-
+			// target sprite. For the original Sopwith destroyed-
+			// building sprite, this evaluates to 2, which is
+			// the original behavior.
+			ic = 110 / TargetExplosionSize(obo->ob_orient);
+		} else {
+			ic = 2;
+		}
 	}
 	mansym = obotype == PLANE
 		 && (obo->ob_state == FLYING || obo->ob_state == WOUNDED);
@@ -723,7 +755,7 @@ void initexpl(OBJECTS *obo, int small)
 		ob->ob_lx = ob->ob_ly = ob->ob_hitcount = ob->ob_speed = 0;
 		ob->ob_owner = obo;
 		ob->ob_clr = oboclr;
-		ob->ob_newsym = &symbol_debris[0]->sym[0];
+		ob->ob_newsym = &symbol_debris[0].sym[0];
 		ob->ob_drawf = dispexpl;
 		ob->ob_movef = moveexpl;
 
@@ -785,7 +817,7 @@ static OBJECTS *initflock(const original_ob_t *orig_ob)
 	ob->ob_orient = 0;
 	ob->ob_life = FLOCKLIFE;
 	ob->ob_owner = ob;
-	ob->ob_newsym = &symbol_flock[0]->sym[0];
+	ob->ob_newsym = &symbol_flock[0].sym[0];
 	ob->ob_drawf = NULL;
 	ob->ob_movef = moveflck;
 	ob->ob_clr = 1;
@@ -803,10 +835,10 @@ static OBJECTS *initflock(const original_ob_t *orig_ob)
 void initbird(OBJECTS *obo, int i)
 {
 	OBJECTS *ob;
-	static int ibx[] = { 8, 3, 0, 6, 7, 14, 10, 12 };
-	static int iby[] = { 16, 1, 8, 3, 12, 10, 7, 14 };
-	static int ibdx[] = { -2, 2, -3, 3, -1, 1, 0, 0 };
-	static int ibdy[] = { -1, -2, -1, -2, -1, -2, -1, -2 };
+	static const int ibx[] = { 8, 3, 0, 6, 7, 14, 10, 12 };
+	static const int iby[] = { 16, 1, 8, 3, 12, 10, 7, 14 };
+	static const int ibdx[] = { -2, 2, -3, 3, -1, 1, 0, 0 };
+	static const int ibdy[] = { -1, -2, -1, -2, -1, -2, -1, -2 };
 
 	ob = allocobj();
 	if (!ob) {
@@ -823,7 +855,7 @@ void initbird(OBJECTS *obo, int i)
 	    0;
 	ob->ob_life = BIRDLIFE;
 	ob->ob_owner = obo;
-	ob->ob_newsym = &symbol_bird[0]->sym[0];
+	ob->ob_newsym = &symbol_bird[0].sym[0];
 	ob->ob_drawf = NULL;
 	ob->ob_movef = movebird;
 	ob->ob_clr = obo->ob_clr;
@@ -852,7 +884,7 @@ static OBJECTS *initox(const original_ob_t *orig_ob)
 	ob->ob_orient = ob->ob_lx = ob->ob_ly = ob->ob_ldx =
 	    ob->ob_ldy = ob->ob_dx = ob->ob_dy = 0;
 	ob->ob_owner = ob;
-	ob->ob_newsym = &symbol_ox[0]->sym[orig_ob->transform];
+	ob->ob_newsym = &symbol_ox[0].sym[orig_ob->transform];
 	ob->ob_drawf = NULL;
 	ob->ob_movef = moveox;
 	ob->ob_clr = 1;
@@ -990,7 +1022,7 @@ void swrestart(void)
 		ob = planes[player];
 		inc = 0;
 
-		get_endlevel(ob);
+		GetEndLevel(ob);
 
 		// Count down the remaining lives; the player gets awarded
 		// extra points for each life.
@@ -1042,39 +1074,47 @@ void swinit(int argc, char *argv[])
 	// longer be necessary.
 	swloadconf();
 
+	// We must generate symbols early, so that custom map files
+	// can replace symbols:
+	GenerateSymbols();
+
 	for (i=1; i<argc; ++i) {
-		if (!strcasecmp(argv[i], "-n")) {
-			n = 1;
+		if (!strcasecmp(argv[i], "-v")
+		 || !strcasecmp(argv[i], "--version")) {
+			puts(PACKAGE_STRING);
+			exit(0);
+		} else if (!strcasecmp(argv[i], "-n")) {
+			n = true;
 		} else if (!strcasecmp(argv[i], "-s")) {
-			s = 1;
+			s = true;
 		} else if (!strcasecmp(argv[i], "-c")) {
-			c = 1;
+			c = true;
 		} else if (!strcasecmp(argv[i], "-f")) {
 			vid_fullscreen = 1;
 		} else if (!strncasecmp(argv[i], "-g", 2)) {
 			sscanf(& (argv[i][2]), "%d", &starting_level);
 			gamenum = starting_level;
 		} else if (!strcasecmp(argv[i], "-q")) {
-			soundflg = 1;
+			soundflg = true;
 		} else if (!strcmp(argv[i], "-m") && (i + 1) < argc) {
-			load_custom_level(argv[i + 1]);
+			LoadCustomLevel(argv[i + 1]);
 			++i;
 		} else
 #ifdef TCPIP
 		if (!strcasecmp(argv[i], "-l")) {
-			a = 1;
+			a = true;
 			asynmode = ASYN_LISTEN;
 		} else if (!strcasecmp(argv[i], "-j")) {
 			if (i + 1 >= argc) {
-				error_exit("insufficient arguments to -j");
+				ErrorExit("insufficient arguments to -j");
 			}
-			a = 1;
+			a = true;
 			asynmode = ASYN_CONNECT;
 			++i;
 			snprintf(asynhost, sizeof(asynhost), "%s", argv[i]);
 		} else if (!strcmp(argv[i], "-p")) {
 			if (i + 1 >= argc) {
-				error_exit("insufficient arguments to -p");
+				ErrorExit("insufficient arguments to -p");
 			}
 			++i;
 			asynport = atoi(argv[i]);
@@ -1096,23 +1136,18 @@ void swinit(int argc, char *argv[])
 
 	initseed();
 
-	// sdh 27/6/2002: generate symbol objects
-
-	symbol_generate();
-
 	Timer_Init();
 	Vid_Init();
 	Vid_SetVideoPalette(conf_video_palette);
 
-	// dont init speaker if started with -q (quiet)
+	// Don't init speaker if started with -q (quiet)
 	if (soundflg) {
-		Speaker_Init();		// init pc speaker
+		Speaker_Init();
 	}
 
 	initsndt();
 	
 	// set playmode if we can, from command line options
-
 	playmode =
 		n ? PLAYMODE_NOVICE :
 		s ? PLAYMODE_SINGLE :
