@@ -1,12 +1,11 @@
 //
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 // Keep track of and display heads-up splats.
@@ -19,22 +18,22 @@
 #include "swsplat.h"
 #include "swsymbol.h"
 
-typedef struct {
+struct splat {
 	int x, y;
 	int clr;
 	sopsym_t *sym;
-} splat_t;
+};
 
 #define MAX_SPLATS 64
 
-static int oxsplatted = 0;
-static splat_t splats[MAX_SPLATS];
+static bool oxsplatted = false;
+static struct splat splats[MAX_SPLATS];
 static int num_splats = 0;
 
 void swclearsplats(void)
 {
 	num_splats = 0;
-	oxsplatted = 0;
+	oxsplatted = false;
 }
 
 void swdispsplats(void)
@@ -72,44 +71,43 @@ static unsigned long randsd(void)
 	return 0;
 }
 
-static void add_splat(splat_t *splat)
+static void AddSplat(struct splat *splat)
 {
 	if (num_splats < MAX_SPLATS) {
-		memcpy(&splats[num_splats], splat, sizeof(splat_t));
+		memcpy(&splats[num_splats], splat, sizeof(struct splat));
 		++num_splats;
 	}
 }
 
-
 void swsplatbird(void)
 {
-	splat_t splat;
+	struct splat splat;
 	
 	randsd();
 
 	splat.x = (unsigned) (seed % (SCR_WDTH - 32));
 	splat.y = (unsigned) (seed % (SCR_HGHT - 60)) + 60;
-	splat.sym = &symbol_birdsplat->sym[0];
+	splat.sym = &symbol_birdsplat[0].sym[0];
 	splat.clr = OWNER_PLAYER2;
 
-	add_splat(&splat);
+	AddSplat(&splat);
 }
 
 void swwindshot(void)
 {
-	splat_t splat;
+	struct splat splat;
 
 	randsd();
 
 	splat.x = (unsigned) (seed % (SCR_WDTH - 16));
 	splat.y = (unsigned) (seed % (SCR_HGHT - 50)) + 50;
-	splat.sym = &symbol_shotwin->sym[0];
+	splat.sym = &symbol_shotwin[0].sym[0];
 	splat.clr = OWNER_PLAYER1;
 
-	add_splat(&splat);
+	AddSplat(&splat);
 }
 
 void swsplatox(void)
 {
-	oxsplatted = 1;
+	oxsplatted = true;
 }
