@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 // Status bar code
@@ -27,7 +26,7 @@ static void dispribbonrow (int *ribbonid, int ribbons_nr, int y)
 	for (i = 0; i < ribbons_nr; i++) {
 		int ribbon_id = ribbonid[i];
 
-		Vid_DispSymbol(offset, y, &symbol_ribbon[ribbon_id]->sym[0],
+		Vid_DispSymbol(offset, y, &symbol_ribbon[ribbon_id].sym[0],
 		               OWNER_PLAYER1);
 		offset += 8;
 	}
@@ -44,7 +43,7 @@ static void dispmedals(OBJECTS *ob)
 		int medal_id = ob->ob_score.medals[i];
 
 		Vid_DispSymbol(medal_offset + medal_offsets[medal_id],
-		               11, &symbol_medal[medal_id]->sym[0],
+		               11, &symbol_medal[medal_id].sym[0],
 		               OWNER_PLAYER1);
 		medal_offset += medal_widths[medal_id];
 	}
@@ -201,7 +200,6 @@ static void dispmap(void)
 	}
 }
 
-
 void dispstatusbar(void)
 {
 	dispmap();
