@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 // Sprites
@@ -17,6 +16,9 @@
 
 #include "swsymbol.h"
 
+// Linked list of all symbols, to allow replacement in custom levels:
+symset_t *all_symsets = NULL;
+
 // Order here is counterintuitive: cyan (1) is brighter than magenta (2):
 static const char *color_chars = " *-#";
 
@@ -62,7 +64,7 @@ static const char *swplnsym[] = {
 	"                - -   * * *     \n"
 	"              -     - * * *     \n"
 	"            -   - - * - -       \n"
-	"                *   - -         \n"
+	"                * * - -         \n"
 	"              * * * -   * *     \n"
 	"            * * * -     * *     \n"
 	"  *   * * * * * *               \n"
@@ -197,7 +199,6 @@ static const char *swwinsym[] = {  /*  Win plane pixel array  */
 	"                                \n",
 };
 
-
 static const char *swbmbsym[] = {
 
 	"                \n"
@@ -219,6 +220,25 @@ static const char *swbmbsym[] = {
 	"      *         \n"
 };
 
+static const char custom_target_sym[] = {
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n",
+};
+
 static const char *swtrgsym[] = {
 
 	"                      -         \n"
@@ -237,7 +257,8 @@ static const char *swtrgsym[] = {
 	"* * - * * * * * * * * * * - * * \n"
 	"* * - * * * * * * * * * * - * * \n"
 	"* * - * * * * * * * * * * - * * \n",
-	// -------------------------------
+
+	// TARGET_FACTORY:
 	"                    - -     - - \n"
 	"                    - -     - - \n"
 	"                    - -     - - \n"
@@ -254,7 +275,8 @@ static const char *swtrgsym[] = {
 	"* * * - * - * - * - * - * * - - \n"
 	"* * * * * * * * * * * * * * - - \n"
 	"* * * * * * * * * * * * * * - - \n",
-	// -------------------------------
+
+	// TARGET_OIL_TANK:
 	"                                \n"
 	"                                \n"
 	"                                \n"
@@ -271,7 +293,8 @@ static const char *swtrgsym[] = {
 	"    - -     -   -       - -     \n"
 	"    - -     - - -       - -     \n"
 	"    - -     -   -       - -     \n",
-	// -------------------------------
+
+	// TARGET_TANK:
 	"                                \n"
 	"                                \n"
 	"                                \n"
@@ -288,25 +311,205 @@ static const char *swtrgsym[] = {
 	"- * * * * * * * * * * * * * * - \n"
 	"- * * * * * * * * * * * * * * - \n"
 	"  - - - - - - - - - - - - - -   \n",
+
+	// TARGET_TRUCK:
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"* # * * # * * # * *             \n"
+	"* # * * # * * # * *             \n"
+	"* # * * # * * # * * - - -       \n"
+	"* # * * # * * # * * - # #       \n"
+	"* # * * # * * # * * - # #       \n"
+	"* # * * # * * # * * - # #       \n"
+	"- - - - - - - - - - * * - - - - \n"
+	"* * * * * * * * * * * - * * * * \n"
+	"- * * - * * - - - - - * * - * * \n"
+	"    - - -               - - -   \n"
+	"      -                   -     \n",
+
+	// TARGET_TANKER_TRUCK:
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"  * * * * * * *                 \n"
+	"* * * - * - * * * - - - -       \n"
+	"* * * - - - * * * - * # #       \n"
+	"* * * - * - * * * - * # #       \n"
+	"* * * - - - * * * - * # #       \n"
+	"  * * - * - * *   - * * * * *   \n"
+	"- - - - - - - - - * * * - - - - \n"
+	"* * * * * * * * * * * - * * * * \n"
+	"- * * - * * - - - - - * * - * * \n"
+	"    - - -               - - -   \n"
+	"      -                   -     \n",
+
+	// TARGET_FLAG:
+	"          -                     \n"
+	"          - * * * * * * *       \n"
+	"          - * * * * * * *       \n"
+	"          - * * * * * * *       \n"
+	"          - * * * * * * *       \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"          -                     \n"
+	"        - - -                   \n"
+	"      - - - - -                 \n",
+
+	// TARGET_TENT:
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"          * * * * * *           \n"
+	"      * * * - - - - * * *       \n"
+	"    * * - - - - - - - - * *     \n"
+	"  # * - - - - - - - - - - * #   \n"
+	"  # * - - - - - - - - - - * #   \n"
+	"#   * - - - - - - - - - - *   # \n"
+	"#   * - - - - - - - - - - *   # \n",
+
+	// User-defined target types:
+	custom_target_sym,  // TARGET_CUSTOM1
+	custom_target_sym,  // TARGET_CUSTOM2
+	custom_target_sym,  // TARGET_CUSTOM3
+	custom_target_sym,  // TARGET_CUSTOM4
+	custom_target_sym,  // TARGET_CUSTOM5
+	custom_target_sym,  // TARGET_CUSTOM_PASSIVE1
+	custom_target_sym,  // TARGET_CUSTOM_PASSIVE2
+	custom_target_sym,  // TARGET_CUSTOM_PASSIVE3
+	custom_target_sym,  // TARGET_CUSTOM_PASSIVE4
+	custom_target_sym,  // TARGET_CUSTOM_PASSIVE5
+
 };
 
-static const char *swhtrsym =
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                                \n"
-"                            *   \n"
-"*                           * * \n"
-"* *       *           -   * * * \n"
-"* *   * * *     * *   - * * * * \n"
-"* * * * - - * * * * - * * * * * \n"
-"* * * * - - * * * - * * * * * * \n";
+static const char destroyed_building[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                            *   \n"
+	"*                           * * \n"
+	"* *       *           -   * * * \n"
+	"* *   * * *     * *   - * * * * \n"
+	"* * * * - - * * * * - * * * * * \n"
+	"* * * * - - * * * - * * * * * * \n";
+
+static const char destroyed_truck[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"    * *   -       * * -         \n"
+	"  * * - * * -   * - - * *       \n"
+	"* * - - - * * - * * * * - -     \n"
+	"- * * - * * * * * * * * * - - * \n";
+
+static const char destroyed_flag[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"        - - -       * * *       \n"
+	"      - - - - - - - - - * *     \n";
+
+static const char destroyed_tent[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"        * * *         * *       \n"
+	"# # * * * * * * * * * * * * #   \n";
+
+static const char destroyed_custom[] =
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"                                \n"
+	"  * * * * * * * * * * * * * * * \n"
+	"  * * * * * * * * * * * * * * * \n";
+
+// There is one entry in this array for each in swtrgsym, so that different
+// targets can have different "destroyed" symbols:
+static const char *swhtrsym[] = {
+	destroyed_building,  // TARGET_HANGAR
+	destroyed_building,  // TARGET_FACTORY
+	destroyed_building,  // TARGET_OIL_TANK
+	destroyed_building,  // TARGET_TANK
+	destroyed_truck,  // TARGET_TRUCK
+	destroyed_truck,  // TARGET_TANKER_TRUCK
+	destroyed_flag,  // TARGET_FLAG
+	destroyed_tent,  // TARGET_TENT
+	destroyed_custom,  // TARGET_CUSTOM1
+	destroyed_custom,  // TARGET_CUSTOM2
+	destroyed_custom,  // TARGET_CUSTOM3
+	destroyed_custom,  // TARGET_CUSTOM4
+	destroyed_custom,  // TARGET_CUSTOM5
+	destroyed_custom,  // TARGET_CUSTOM_PASSIVE1
+	destroyed_custom,  // TARGET_CUSTOM_PASSIVE2
+	destroyed_custom,  // TARGET_CUSTOM_PASSIVE3
+	destroyed_custom,  // TARGET_CUSTOM_PASSIVE4
+	destroyed_custom,  // TARGET_CUSTOM_PASSIVE5
+};
 
 static const char *swexpsym[] = {
 	"* * *   -       \n"
@@ -463,25 +666,26 @@ static const char *swoxsym[] = {
 	"# - - # # - - # # -   - - #     \n"
 };
 
-static const char *swshtsym =
-"                            *   \n"
-"      *                 * *     \n"
-"      *           * * *         \n"
-"        *       *               \n"
-"        *     *     * *         \n"
-"          * * *   *     *       \n"
-"* * * *   * * - *         * *   \n"
-"        * * * - - *           * \n"
-"        * * * - * *             \n"
-"          * - - *   * *         \n"
-"        *   * *         *       \n"
-"      *       *         *       \n"
-"    *         *         *       \n"
-"    *     * * *       *         \n"
-"    *   *               *       \n"
-"          * *             *     \n";
+static const char *swshtsym[] = {
+	"                            *   \n"
+	"      *                 * *     \n"
+	"      *           * * *         \n"
+	"        *       *               \n"
+	"        *     *     * *         \n"
+	"          * * *   *     *       \n"
+	"* * * *   * * - *         * *   \n"
+	"        * * * - - *           * \n"
+	"        * * * - * *             \n"
+	"          * - - *   * *         \n"
+	"        *   * *         *       \n"
+	"      *       *         *       \n"
+	"    *         *         *       \n"
+	"    *     * * *       *         \n"
+	"    *   *               *       \n"
+	"          * *             *     \n"
+};
 
-static const char *swsplsym =
+static const char *swsplsym[] = {
 "                                  - - -                         \n"
 "                                  - * * - -                     \n"
 "      *                             - * * * -             *     \n"
@@ -513,7 +717,8 @@ static const char *swsplsym =
 "      *         - - - - -         -         - - - - - * * * * - \n"
 "  * *       *       -       - - - - - -               - * * * - \n"
 "* * *     * * *     -           -         *             - * -   \n"
-"* *         *                 -                 *         -     \n";
+"* *         *                 -                 *         -     \n"
+};
 
 static const char *swmscsym[] = {
 
@@ -635,7 +840,7 @@ static const char *swribbonsym[] = {
 	"- - # # # - -   \n", /* MMWWWMM : PREVALOUR */
 };
 
-static void rotate(int *x, int *y, int w, int h, int rotations, bool mirror)
+static void Rotate(int *x, int *y, int w, int h, int rotations, bool mirror)
 {
 	int i, tmp;
 
@@ -649,8 +854,8 @@ static void rotate(int *x, int *y, int w, int h, int rotations, bool mirror)
 	}
 }
 
-static void sopsym_from_text(sopsym_t *sym, const char *text, int w, int h,
-                             int rotations, bool mirror)
+static void SopsymFromText(sopsym_t *sym, const char *text, int w, int h,
+                           int rotations, bool mirror)
 {
 	const char *p, *p2;
 	int x, y, dx, dy, c;
@@ -679,7 +884,7 @@ static void sopsym_from_text(sopsym_t *sym, const char *text, int w, int h,
 
 			if (x < w * 2 && y < h && (x % 2) == 0) {
 				dx = x / 2; dy = y;
-				rotate(&dx, &dy, w, h, rotations, mirror);
+				Rotate(&dx, &dy, w, h, rotations, mirror);
 				sym->data[dy * sym->w + dx] = c;
 			}
 			x++;
@@ -687,42 +892,60 @@ static void sopsym_from_text(sopsym_t *sym, const char *text, int w, int h,
 	}
 }
 
-static symset_t *symset_from_text(const char *text, int w, int h)
+void SymsetFromText(symset_t *s, const char *text, int w, int h)
 {
-	symset_t *s = malloc(sizeof(*s));
 	int r;
 
 	for (r = 0; r < 4; r++)
 	{
-		sopsym_from_text(&s->sym[r], text, w, h, r, false);
-		sopsym_from_text(&s->sym[r + 4], text, w, h, r, true);
+		SopsymFromText(&s->sym[r], text, w, h, r, false);
+		SopsymFromText(&s->sym[r + 4], text, w, h, r, true);
+	}
+}
+
+symset_t *LookupSymset(const char *name, int frame)
+{
+	symset_t *s;
+
+	for (s = all_symsets; s != NULL; s = s->next) {
+		if (!strcmp(name, s->name) && frame == s->frame) {
+			return s;
+		}
 	}
 
-	return s;
+	return NULL;
+}
+
+static void InitSymset(symset_t *s, const char *name, int frame)
+{
+	s->name = name;
+	s->frame = frame;
+	s->next = all_symsets;
+	all_symsets = s;
 }
 
 // converted symbols:
 
-symset_t *symbol_bomb[2];                 // swbmbsym
-symset_t *symbol_targets[4];              // swtrgsym
-symset_t *symbol_target_hit;              // swhtrsym
-symset_t *symbol_debris[8];               // swexpsym
-symset_t *symbol_flock[2];                // swflksym
-symset_t *symbol_bird[2];                 // swbrdsym
-symset_t *symbol_ox[2];                   // swoxsym
-symset_t *symbol_shotwin;                 // swshtsym
-symset_t *symbol_birdsplat;               // swsplsym
-symset_t *symbol_missile[4];              // swmscsym
-symset_t *symbol_burst[2];                // swbstsym
-symset_t *symbol_plane[4];                // swplnsym
-symset_t *symbol_plane_hit[2];            // swhitsym
-symset_t *symbol_plane_win[4];            // swwinsym
-symset_t *symbol_medal[3];                // swmedalsym
-symset_t *symbol_ribbon[6];               // swribbonsym
+symset_t symbol_bomb[2];                 // swbmbsym
+symset_t symbol_targets[NUM_TARGET_TYPES];    // swtrgsym
+symset_t symbol_target_hit[NUM_TARGET_TYPES]; // swhtrsym
+symset_t symbol_debris[8];               // swexpsym
+symset_t symbol_flock[2];                // swflksym
+symset_t symbol_bird[2];                 // swbrdsym
+symset_t symbol_ox[2];                   // swoxsym
+symset_t symbol_shotwin[1];              // swshtsym
+symset_t symbol_birdsplat[1];            // swsplsym
+symset_t symbol_missile[4];              // swmscsym
+symset_t symbol_burst[2];                // swbstsym
+symset_t symbol_plane[4];                // swplnsym
+symset_t symbol_plane_hit[2];            // swhitsym
+symset_t symbol_plane_win[4];            // swwinsym
+symset_t symbol_medal[3];                // swmedalsym
+symset_t symbol_ribbon[6];               // swribbonsym
 
 // special symbol for single pixel (bullets etc)
 
-static unsigned char pixel_data[] = { 3 };
+static uint8_t pixel_data[] = { 3 };
 
 sopsym_t symbol_pixel = {
 	pixel_data,
@@ -731,31 +954,32 @@ sopsym_t symbol_pixel = {
 };
 
 // generate array of symset_t structs from array of strings:
-#define symsets_from_text(text, w, h, out)                        \
+#define SYMSETS_FROM_TEXT(text, w, h, out)                        \
         { int _i;                                                 \
-          for (_i=0; _i<sizeof(out)/sizeof(*(out)); ++_i)         \
-             (out)[_i] = symset_from_text((text)[_i], (w), (h));  \
+          for (_i = 0; _i < arrlen(out); ++_i) {                  \
+             InitSymset(&(out)[_i], #text, _i);                   \
+             SymsetFromText(&(out)[_i], (text)[_i], (w), (h));    \
+          }                                                       \
         }
 
-void symbol_generate(void)
+void GenerateSymbols(void)
 {
-	symsets_from_text(swbmbsym, 8, 8, symbol_bomb);
-	symsets_from_text(swtrgsym, 16, 16, symbol_targets);
-	symsets_from_text(swexpsym, 8, 8, symbol_debris);
-	symsets_from_text(swflksym, 16, 16, symbol_flock);
-	symsets_from_text(swbrdsym, 4, 2, symbol_bird);
-	symsets_from_text(swoxsym, 16, 16, symbol_ox);
-	symsets_from_text(swmscsym, 8, 8, symbol_missile);
-	symsets_from_text(swbstsym, 8, 8, symbol_burst);
-	symsets_from_text(swplnsym, 16, 16, symbol_plane);
-	symsets_from_text(swhitsym, 16, 16, symbol_plane_hit);
-	symsets_from_text(swwinsym, 16, 16, symbol_plane_win);
-	symsets_from_text(swmedalsym, 8, 12, symbol_medal);
-	symsets_from_text(swribbonsym, 8, 2, symbol_ribbon);
-
-	symbol_target_hit = symset_from_text(swhtrsym, 16, 16);
-	symbol_shotwin = symset_from_text(swshtsym, 16, 16);
-	symbol_birdsplat = symset_from_text(swsplsym, 32, 32);
+	SYMSETS_FROM_TEXT(swbmbsym, 8, 8, symbol_bomb);
+	SYMSETS_FROM_TEXT(swtrgsym, 16, 16, symbol_targets);
+	SYMSETS_FROM_TEXT(swexpsym, 8, 8, symbol_debris);
+	SYMSETS_FROM_TEXT(swflksym, 16, 16, symbol_flock);
+	SYMSETS_FROM_TEXT(swbrdsym, 4, 2, symbol_bird);
+	SYMSETS_FROM_TEXT(swoxsym, 16, 16, symbol_ox);
+	SYMSETS_FROM_TEXT(swmscsym, 8, 8, symbol_missile);
+	SYMSETS_FROM_TEXT(swbstsym, 8, 8, symbol_burst);
+	SYMSETS_FROM_TEXT(swplnsym, 16, 16, symbol_plane);
+	SYMSETS_FROM_TEXT(swhitsym, 16, 16, symbol_plane_hit);
+	SYMSETS_FROM_TEXT(swwinsym, 16, 16, symbol_plane_win);
+	SYMSETS_FROM_TEXT(swmedalsym, 8, 12, symbol_medal);
+	SYMSETS_FROM_TEXT(swribbonsym, 8, 2, symbol_ribbon);
+	SYMSETS_FROM_TEXT(swhtrsym, 16, 16, symbol_target_hit);
+	SYMSETS_FROM_TEXT(swshtsym, 16, 16, symbol_shotwin);
+	SYMSETS_FROM_TEXT(swsplsym, 32, 32, symbol_birdsplat);
 }
 
 //
