@@ -2,12 +2,11 @@
 // Copyright(C) 1984-2000 David L. Clark
 // Copyright(C) 2001-2005 Simon Howard
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2 of the License, or (at your
-// option) any later version. This program is distributed in the hope that
-// it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// You can redistribute and/or modify this program under the terms of the
+// GNU General Public License version 2 as published by the Free Software
+// Foundation, or any later version. This program is distributed WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.
 //
 //
 //        swtext - text processing.. input/output
@@ -33,7 +32,7 @@ static int cur_color;		// text color
 
 static inline void drawchar(int x, int y, int c)
 {
-	unsigned char *p;
+	uint8_t *p;
 	int x2, y2;
 
 	if (c < 0 || c >= 256) {
@@ -61,7 +60,6 @@ static inline void drawchar(int x, int y, int c)
 	}
 }
 
-
 void swputc(char c)
 {
 	if (isprint(c)) {
